@@ -48,23 +48,15 @@ impl<T, const N: usize> AsRef<[T]> for StaticVec<T, { N }> {
 impl<T: Clone, const N: usize> Clone for StaticVec<T, { N }> {
   #[inline]
   default fn clone(&self) -> Self {
-    Self {
-      data: {
-        unsafe {
-          let mut data: [MaybeUninit<T>; N] = MaybeUninit::uninit_array();
-          for i in 0..self.length {
-            // Put the clones in a separate stack-allocated array first, so we're
-            // not forced to increment `length` for the result variable one by one in order to
-            // account for the possibility of a `clone` call panicking.
-            data
-              .get_unchecked_mut(i)
-              .write(self.get_unchecked(i).clone());
-          }
-          data
-        }
-      },
-      length: self.length,
+    let mut res = Self::new();
+    for item in self {
+      // Safety: res and self have the same type, so they're guaranteed to
+      // have the same capacity, and push_unchecked will never overflow.
+      unsafe {
+        res.push_unchecked(item.clone());
+      }
     }
+    res
   }
 
   #[inline]
@@ -92,15 +84,33 @@ impl<T: Clone, const N: usize> Clone for StaticVec<T, { N }> {
 impl<T: Copy, const N: usize> Clone for StaticVec<T, { N }> {
   #[inline(always)]
   fn clone(&self) -> Self {
-    // The code this generates is *way* more efficient than `copy_to_nonoverlapping`,
-    // and safe in this case because we're transmuting between two instances of the same type.
-    unsafe { core::mem::transmute_copy(self) }
+    match self.length {
+      // If `self` is empty, just return a new StaticVec.
+      0 => Self::new(),
+      _ => {
+        // Create an uninitialized StaticVec (which allows the compiler to do better return value
+        // optimization when we call `assume_init` on it later, rather than fully instantiating it
+        // now.)
+        let mut res = MaybeUninit::<Self>::uninit();
+        // Copy a usize worth of bytes to `res` for `length`, and then `self.length` times
+        // `size_of::<T>` bytes to `res` for `data`. Note that the order we do this in does
+        // not actually matter, it just matters that the total number of bytes copied is exactly
+        // correct (which we can be confident is true by using the next two functions one
+        // after another.)
+        unsafe {
+          self.copy_length_to(&mut res);
+          self.copy_items_to(&mut res);
+          res.assume_init()
+        }
+      }
+    }
   }
 
   #[inline(always)]
   fn clone_from(&mut self, rhs: &Self) {
-    // See above comment for safety note.
-    *self = unsafe { core::mem::transmute_copy(rhs) }
+    // Here we take advantage of the above efficient `clone` implementation,
+    // in reverse.
+    *self = rhs.clone();
   }
 }
 
@@ -379,7 +389,7 @@ impl<const N: usize> Read for StaticVec<u8, { N }> {
     // have been written. This means that when we perform the shift at the
     // end, self.length is already correct.
     for buf in bufs {
-      if self.length == 0 {
+      if self.is_empty() {
         break;
       }
 
